<?php

namespace XF\ConnectedAccount\Provider;

use XF\Entity\ConnectedAccountProvider;

class XenForo extends AbstractProvider
{
	public function getOAuthServiceName(): string
	{
		return 'XF:Service\XenForo';
	}

	public function getProviderDataClass(): string
	{
		return 'XF:ProviderData\XenForo';
	}

	public function getDefaultOptions(): array
	{
		return [
			'display_title' => '',
			'board_url' => '',
			'client_id' => '',
			'client_secret' => ''
		];
	}

	public function getOAuthConfig(ConnectedAccountProvider $provider, $redirectUri = null): array
	{
		return [
			'key' => $provider->options['client_id'],
			'secret' => $provider->options['client_secret'],
			'scopes' => ['user:read'],
			'redirect' => $redirectUri ?: $this->getRedirectUri($provider)
		];
	}

	public function getTitle(): \XF\Phrase
	{
		return \XF::phrase('con_acc.xenforo');
	}

	public function getDescription(): \XF\Phrase
	{
		return \XF::phrase('con_acc_desc.xenforo');
	}

	public function renderConfig(ConnectedAccountProvider $provider): string
	{
		return \XF::app()->templater()->renderTemplate('admin:connected_account_provider_xenforo', [
			'options' => $this->getEffectiveOptions($provider->options)
		]);
	}

	public function getTestTemplateName(): string
	{
		return 'admin:connected_account_provider_test_xenforo';
	}

	public function renderAssociated(ConnectedAccountProvider $provider, \XF\Entity\User $user): string
	{
		return \XF::app()->templater()->renderTemplate('public:connected_account_associated_xenforo', [
			'provider' => $provider,
			'user' => $user,
			'providerData' => $provider->getUserInfo($user),
			'connectedAccounts' => $user->Profile->connected_accounts
		]);
	}
}