<?php

namespace XF\ControllerPlugin;

use function call_user_func_array;

/**
 * @property-read \XF\App $app
 * @property-read \XF\Mvc\Entity\Manager $em
 * @property-read \XF\Http\Request $request
 * @property-read \XF\Session\Session $session
 *
 * @property string $responseType
 * @property ?string $sectionContext
 *
 * @mixin \XF\Mvc\Controller
 */
abstract class AbstractPlugin
{
	/**
	 * @var \XF\Mvc\Controller
	 */
	protected $controller;

	public function __construct(\XF\Mvc\Controller $controller)
	{
		$this->controller = $controller;
	}

	public function __get($key)
	{
		switch ($key)
		{
			case 'app': return $this->controller->app();
			case 'em': return $this->controller->em();
			case 'request': return $this->controller->request();
			case 'session': return $this->controller->session();
			case 'responseType': return $this->controller->responseType();
			case 'sectionContext': return $this->controller->sectionContext();
		}

		return $this->controller->$key;
	}

	public function __set($key, $value)
	{
		switch ($key)
		{
			case 'responseType': $this->controller->setResponseType($value); return;
			case 'sectionContext': $this->controller->setSectionContext($value); return;
		}

		$this->controller->$key = $value;
	}

	public function __call($method, array $args)
	{
		return call_user_func_array([$this->controller, $method], $args);
	}
}