<?php

namespace XF\Entity;

use XF\Mvc\Entity\Structure;

trait EmbedResolverTrait
{
	public function canViewEmbed(): bool
	{
		$handler = $this->getEmbedHandler();

		if (!$handler)
		{
			return false;
		}

		return $handler->canViewContent($this);
	}

	public function renderEmbed(): string
	{
		$handler = $this->getEmbedHandler();

		if (!$handler)
		{
			return '';
		}

		return $handler->render($this);
	}

	public function getOembedOutput(): array
	{
		$handler = $this->getEmbedHandler();

		if (!$handler)
		{
			return [];
		}

		return $handler->getOembedOutput($this);
	}

	public function getEmbedCodeHtml(): string
	{
		$handler = $this->getEmbedHandler();

		if (!$handler)
		{
			return '';
		}

		return $handler->getEmbedCodeHtml($this);
	}

	public function getOembedEndpointUrl(): string
	{
		if (!method_exists($this, 'getContentUrl'))
		{
			return '';
		}

		return \XF::app()->router('api')->buildLink('canonical:oembed', null, ['url' => $this->getContentUrl(true)]);
	}

	public function getEmbedHandler(): \XF\EmbedResolver\AbstractHandler
	{
		return $this->getEmbedRepo()->getEmbedHandler($this->getEntityContentType());
	}

	public static function addEmbedResolverStructureElements(Structure $structure) {}

	/**
	 * @return \XF\Mvc\Entity\Repository|\XF\Repository\EmbedResolver
	 */
	protected function getEmbedRepo(): \XF\Repository\EmbedResolver
	{
		return $this->repository('XF:EmbedResolver');
	}
}