<?php

namespace XF\FeaturedContent;

use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Entity;

class Thread extends AbstractHandler
{
	public function getContentImage(
		Entity $content,
		?string $sizeCode = null
	): ?string
	{
		/** @var \XF\Entity\Thread $content */
		return $content->getCoverImage();
	}

	public function getContentSnippet(Entity $content): string
	{
		/** @var \XF\Entity\Thread $content */
		return $this->getSnippetFromString($content->FirstPost->message ?? null);
	}

	public function getContentStructuredData(Entity $content): array
	{
		/** @var \XF\Entity\Thread $content */
		return $content->getLdStructuredData($content->FirstPost);
	}

	public function shouldAutoFeature(Entity $content): bool
	{
		/** @var \XF\Entity\Thread $content */
		return $content->Forum->auto_feature ?? false;
	}

	public function getEntityWith(): array
	{
		$visitor = \XF::visitor();

		return [
			'Forum',
			'Forum.Node.Permissions|' . $visitor->permission_combination_id,
			'User',
			'FirstPost'
		];
	}

	protected function addAttachmentsToContent(
		AbstractCollection $imagelessContent,
		AbstractCollection $content
	): void
	{
		$firstPosts = $imagelessContent->pluckNamed('FirstPost', 'first_post_id');

		/** @var \XF\Repository\Attachment $attachmentRepo */
		$attachmentRepo = \XF::repository('XF:Attachment');
		$attachmentRepo->addAttachmentsToContent($firstPosts, 'post');
	}
}
