<?php

namespace XF\Finder;

use XF\Mvc\Entity\Finder;

/**
 * @method \XF\Mvc\Entity\AbstractCollection<\XF\Entity\ProfilePostComment> fetch($limit = null, $offset = null)
 * @method \XF\Entity\ProfilePostComment|null fetchOne($limit = null, $offset = null)
 * @extends \XF\Mvc\Entity\Finder<\XF\Entity\ProfilePostComment>
 */
class ProfilePostComment extends Finder
{
	public function forProfilePost(\XF\Entity\ProfilePost $profilePost, array $limits = [])
	{
		$limits = array_replace([
			'visibility' => true,
			'allowOwnPending' => true
		], $limits);

		$this->where('profile_post_id', $profilePost->profile_post_id);

		if ($limits['visibility'])
		{
			$this->applyVisibilityChecksForProfilePost($profilePost, $limits['allowOwnPending']);
		}

		return $this;
	}

	public function applyVisibilityChecksForProfilePost(\XF\Entity\ProfilePost $profilePost, $allowOwnPending = true)
	{
		$conditions = [];
		$viewableStates = ['visible'];

		if ($profilePost->canViewDeletedComments())
		{
			$viewableStates[] = 'deleted';
			$this->with('DeletionLog');
		}

		$visitor = \XF::visitor();
		if ($profilePost->canViewModeratedComments())
		{
			$viewableStates[] = 'moderated';
		}
		else if ($visitor->user_id && $allowOwnPending)
		{
			$conditions[] = [
				'message_state' => 'moderated',
				'user_id' => $visitor->user_id
			];
		}

		$conditions[] = ['message_state', $viewableStates];

		$this->whereOr($conditions);

		return $this;
	}

	public function newerThan($date)
	{
		$this->where('comment_date', '>', $date);

		return $this;
	}
}