<?php

namespace XF\Job;

use XF\IndexNow\Api;
use XF\Mvc\Entity\Entity;

use function get_class;

class ContentIndexNow extends AbstractJob
{
	use Retryable;

	protected $defaultData = [
		'content_type' => null,
		'content_id' => null,
		'content_url' => null
	];

	public function run($maxRunTime): JobResult
	{
		$options = \XF::options();

		if (!$options->useFriendlyUrls || !$options->indexNow['enabled'])
		{
			return $this->complete();
		}

		if ((!$this->data['content_type'] || !$this->data['content_id']) && !$this->data['content_url'])
		{
			throw new \InvalidArgumentException('Cannot make a request to IndexNow without a valid content type and ID or URL.');
		}

		$contentType = $this->data['content_type'];
		$contentId = $this->data['content_id'];
		$contentUrl = $this->data['content_url'];

		if (!$contentUrl)
		{
			/** @var Entity $content */
			$content = $this->app->findByContentType($contentType, $contentId);

			if (!$content instanceof Entity)
			{
				throw new \InvalidArgumentException('Cannot make a request to IndexNow without a valid content entity.');
			}

			if (!$content instanceof \XF\Entity\LinkableInterface)
			{
				throw new \LogicException(
					'Implement XF\Entity\LinkableInterface for ' . get_class($content)
				);
			}

			$contentUrl = $content->getContentUrl(true);
		}

		$api = new Api();
		$result = $api->index($contentUrl);

		if (!$result)
		{
			$this->attemptLaterOrComplete();
		}

		return $this->complete();
	}
}