<?php

namespace XF\Option;

class IndexNow extends AbstractOption
{
	protected static function canUseIndexNow(&$error = null): bool
	{
		$options = \XF::options();
		if (!$options->useFriendlyUrls)
		{
			$error = \XF::phrase('friendly_urls_must_be_enabled_to_use_indexnow');
			return false;
		}

		return true;
	}

	public static function renderOption(\XF\Entity\Option $option, array $htmlParams): string
	{
		$canUseIndexNow = self::canUseIndexNow($error);

		return self::getTemplate('admin:option_template_indexNow', $option, $htmlParams, [
			'canUseIndexNow' => $canUseIndexNow,
			'error' => $error
		]);
	}

	public static function verifyOption(&$value, \XF\Entity\Option $option): bool
	{
		if (empty($value['enabled']))
		{
			if ($option->option_value['enabled'])
			{
				$value['key'] = false;
			}

			return true;
		}

		$options = \XF::options();
		if (!$option->option_value['enabled'] && $options->useFriendlyUrls)
		{
			$value['key'] = str_replace('_', '', \XF::generateRandomString(32));
		}

		return true;
	}
}