<?php

namespace XF\Repository;

use XF\FeaturedContent\AbstractHandler;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Finder;
use XF\Mvc\Entity\Repository;

class FeaturedContent extends Repository
{
	public function findFeaturedContent(bool $visibleOnly = true): Finder
	{
		$finder = $this->finder(\XF\Finder\FeaturedContent::class)
			->setDefaultOrder('feature_date', 'desc');

		if ($visibleOnly)
		{
			$finder->where('content_visible', true);
		}

		return $finder;
	}

	public function addContentToFeatures(AbstractCollection $features)
	{
		/** @var \XF\Entity\FeaturedContent[] $features */
		$contentMap = [];
		foreach ($features AS $featureId => $feature)
		{
			$contentType = $feature->content_type;
			$contentId = $feature->content_id;
			if (!isset($contentMap[$contentType]))
			{
				$contentMap[$contentType] = [];
			}

			$contentMap[$contentType][$featureId] = $contentId;
		}

		foreach ($contentMap AS $contentType => $contentIds)
		{
			$handler = $this->getFeatureHandler($contentType);
			if (!$handler)
			{
				continue;
			}

			$contents = $handler->getContent($contentIds);
			foreach ($contentIds AS $featureId => $contentId)
			{
				$feature = $features[$featureId];
				$content = $contents[$contentId] ?? null;
				$feature->setContent($content);
			}
		}
	}

	public function getFeatureHandler(
		string $contentType,
		bool $throw = false
	): ?AbstractHandler
	{
		$handlerClass = $this->app()->getContentTypeFieldValue(
			$contentType,
			'featured_content_handler_class'
		);
		if (!$handlerClass)
		{
			if ($throw)
			{
				throw new \InvalidArgumentException(
					"No featured content handler for '{$contentType}'"
				);
			}

			return null;
		}

		if (!class_exists($handlerClass))
		{
			if ($throw)
			{
				throw new \InvalidArgumentException(
					"Featured content handler does not exist for '{$contentType}' ({$handlerClass})"
				);
			}

			return null;
		}

		$handlerClass = \XF::extendClass($handlerClass);
		return new $handlerClass($contentType);
	}

	/**
	 * @return string[]
	 */
	public function getSupportedContentTypes(): array
	{
		return array_keys(
			$this->app()->getContentTypeField('featured_content_handler_class')
		);
	}
}
