<?php

namespace XF\Repository;

use XF\Mvc\Entity\Repository;

class Passkey extends Repository
{
	public function getExistingCredentialsForUser($user = null): array
	{
		if ($user === null)
		{
			$user = \XF::visitor();
		}

		$passkeys = $this->finder(\XF\Finder\Passkey::class)
			->where('user_id', $user->user_id)
			->fetch();

		$credentials = [];

		foreach ($passkeys AS $passkey)
		{
			$credentials[] = $passkey->credential_id;
		}

		return $credentials;
	}

	public function findPasskeysForUser(\XF\Entity\User $user): \XF\Mvc\Entity\Finder
	{
		return $this->finder(\XF\Entity\Passkey::class)
			->where('user_id', $user->user_id)
			->order('last_use_date', 'ASC');
	}

	public function updateUserPasskeyProvider(\XF\Entity\User $user): bool
	{
		$passkeys = $this->findPasskeysForUser($user)->fetch();

		/** @var \XF\Entity\TfaProvider $provider */
		$provider = \XF::em()->find(\XF\Entity\TfaProvider::class, 'passkey');

		if ($passkeys->count() === 0)
		{
			/** @var \XF\Entity\UserTfa|null $userTfa */
			$userTfa = $provider->UserEntries[$user->user_id];
			if ($userTfa)
			{
				$userTfa->delete();
			}
		}
		else
		{
			/** @var \XF\Entity\UserTfa $userTfa */
			$userTfa = $provider->UserEntries[$user->user_id];
			if (!$userTfa)
			{
				$tfaRepo = $this->repository(\XF\Repository\Tfa::class);
				$tfaRepo->enableUserTfaProvider($user, $provider, []);
			}
		}

		return true;
	}
}