<?php

namespace XF\Search\Data;

use XF\Mvc\Entity\Entity;
use XF\Search\IndexRecord;
use XF\Search\MetadataStructure;

class Conversation extends AbstractData
{
	public function getIndexData(Entity $entity): IndexRecord
	{
		/** @var \XF\Entity\ConversationMaster $entity */
		$firstMessage = $entity->FirstMessage;

		return IndexRecord::create('conversation', $entity->conversation_id, [
			'title' => $entity->title_,
			'message' => $firstMessage ? $firstMessage->message_ : '',
			'date' => $entity->start_date,
			'user_id' => $entity->user_id,
			'discussion_id' => $entity->conversation_id,
			'metadata' => $this->getMetaData($entity)
		]);
	}

	/**
	 * @return array<string, mixed>
	 */
	protected function getMetaData(\XF\Entity\ConversationMaster $entity): array
	{
		$recipients = array_keys($entity->recipients);
		$recipients[]  = $entity->user_id;

		return [
			'conversation' => $entity->conversation_id,
			'recipients'   => $recipients,
		];
	}

	public function setupMetadataStructure(MetadataStructure $structure): void
	{
		$structure->addField('conversation', MetadataStructure::INT);
		$structure->addField('recipients', MetadataStructure::INT);
	}

	public function getResultDate(Entity $entity): int
	{
		/** @var \XF\Entity\ConversationMaster $entity */
		return $entity->start_date;
	}

	/**
	 * @return array<string, mixed>
	 */
	public function getTemplateData(Entity $entity, array $options = []): array
	{
		return [
			'conversation' => $entity,
			'options' => $options,
		];
	}

	/**
	 * @param bool $forView
	 *
	 * @return list<string>
	 */
	public function getEntityWith($forView = false): array
	{
		$with = ['FirstMessage'];

		if ($forView)
		{
			$with[] = 'Starter';
			$with[] = 'Users|' . \XF::visitor()->user_id;
		}

		return $with;
	}

	/**
	 * @param string|null $error
	 */
	public function canUseInlineModeration(Entity $entity, &$error = null): bool
	{
		return true;
	}
}