<?php

namespace XF\Tfa;

use XF\Entity\TfaProvider;

class Passkey extends AbstractProvider
{
	public function generateInitialData(\XF\Entity\User $user, array $config = []): array
	{
		return [];
	}

	public function trigger($context, \XF\Entity\User $user, array &$config, \XF\Http\Request $request): array
	{
		$passkey = \XF::service(\XF\Service\Passkey\Manager::class);
		$passkey->saveStateToSession(\XF::session());

		return [];
	}

	public function render($context, \XF\Entity\User $user, array $config, array $triggerData): string
	{
		$passkey = \XF::service(\XF\Service\Passkey\Manager::class, \XF::session());

		$passkeyRepo = \XF::repository(\XF\Repository\Passkey::class);
		$existingCredentials = $passkeyRepo->getExistingCredentialsForUser($user);

		$params = [
			'user' => $user,
			'config' => $config,
			'context' => $context,
			'provider' => $this->getProvider(),
			'passkey' => $passkey,
			'existingCredentials' => $existingCredentials
		];
		return \XF::app()->templater()->renderTemplate('public:two_step_passkey', $params);
	}

	public function verify($context, \XF\Entity\User $user, array &$config, \XF\Http\Request $request): bool
	{
		if ($context !== 'login')
		{
			// passkeys are created through the account security page
			return false;
		}

		$session = \XF::session();
		$passkey = \XF::service(\XF\Service\Passkey\Manager::class, $session);

		if (!$passkey->validate($request, $error))
		{
			$passkey->clearStateFromSession($session);
			return false;
		}

		return true;
	}

	public function canDisable(): bool
	{
		return false;
	}

	public function canEnable()
	{
		return false;
	}

	public function canManage(): bool
	{
		return true;
	}

	public function handleManage(\XF\Mvc\Controller $controller, TfaProvider $provider, \XF\Entity\User $user, array $config): ?\XF\Mvc\Reply\AbstractReply
	{
		return $controller->redirect($controller->buildLink('account/security'));
	}
}